import os
import json
import shutil
import re
import glob
from VIStk.Structures.VINFO import *
from tkinter import *
import importlib.util
from pathlib import Path
import sys
import gc
import os

class Screen(VINFO):
    """A VIS screen object
    """
    def __init__(self,name:str,script:str,release:bool=False,icon:str=None,exists:bool=True,desc:str=None):
        super().__init__()
        self.name=name
        self.script=script
        self.release=release
        self.icon=icon
        self.path = self.p_screens+"/"+self.name
        self.m_path = self.p_modules+"/"+self.name

        if not exists:
            with open(self.p_sinfo,"r") as f:
                info = json.load(f)

            info[self.title]["Screens"][self.name] = {"script":script,"release":release}
            if not icon == None:
                info[self.title]["Screens"][self.name]["icon"] = icon
            
            if not desc == None:
                info[self.title]["Screens"][self.name]["desc"] = desc
            else:
                info[self.title]["Screens"][self.name]["desc"] = "A VIS Created Executable"

            info[self.title]["Screens"][self.name]["version"] = "1.0.0"#always making first major version of screen

            info[self.title]["Screens"][self.name]["current"] = None#always making first major version of screen

            with open(self.p_sinfo,"w") as f:
                json.dump(info,f,indent=4)

            shutil.copyfile(self.p_templates+"/screen.txt",self.p_project+"/"+script)
            os.mkdir(self.p_screens+"/"+self.name)
            os.mkdir(self.p_modules+"/"+self.name)

        with open(self.p_sinfo,"r") as f:
                info = json.load(f)
        self.desc = info[self.title]["Screens"][self.name]["desc"]
        self.s_version = info[self.title]["Screens"][self.name]["version"]
        self.current = info[self.title]["Screens"][self.name]["current"]
        
    def addElement(self,element:str) -> int:
        if validName(element):
            if not os.path.exists(self.path+"/f_"+element+".py"):
                shutil.copyfile(self.p_templates+"/f_element.txt",self.path+"/f_"+element+".py")
                print(f"Created element f_{element}.py in {self.path}")
                self.patch(element)
            if not os.path.exists(self.m_path+"/m_"+element+".py"):
                with open(self.m_path+"/m_"+element+".py", "w"): pass
                print(f"Created module m_{element}.py in {self.m_path}")
            return 1
        else:
            return 0
        
    def addMenu(self,menu:str) -> int:
        pass #will be command line menu creation tool

    def patch(self,element:str) -> int:
        """Patches up the template after its copied
        """
        if os.path.exists(self.path+"/f_"+element+".py"):
            with open(self.path+"/f_"+element+".py","r") as f:
                text = f.read()
            text = text.replace("<frame>","f_"+element)
            with open(self.path+"/f_"+element+".py","w") as f:
                f.write(text)
            print(f"patched f_{element}.py")
            return 1
        else:
            print(f"Could not patch, element does not exist.")
            return 0
    
    def stitch(self) -> int:
        """Connects screen elements to a screen
        """
        with open(self.p_project+"/"+self.script,"r") as f: text = f.read()
        stitched = []
        #Elements
        pattern = r"#%Screen Elements.*#%Screen Grid"

        elements = glob.glob(self.path+'/f_*')#get all elements
        for i in range(0,len(elements),1):#iterate into module format
            elements[i] = elements[i].replace("\\","/")
            elements[i] = elements[i].replace(self.path+"/","Screens."+self.name+".")[:-3]
            stitched.append(elements[i])
        #combine and change text
        elements = "from " + " import *\nfrom ".join(elements) + " import *\n"
        text = re.sub(pattern, "#%Screen Elements\n" + elements + "\n#%Screen Grid", text, flags=re.DOTALL)

        #Modules
        pattern = r"#%Screen Modules.*#%Handle Arguments"

        modules = glob.glob(self.m_path+'/m_*')#get all modules
        for i in range(0,len(modules),1):#iterate into module format
            modules[i] = modules[i].replace("\\","/")
            modules[i] = modules[i].replace(self.m_path+"/","modules."+self.name+".")[:-3]
            stitched.append(modules[i])
        #combine and change text
        modules = "from " + " import *\nfrom ".join(modules) + " import *\n"
        text = re.sub(pattern, "#%Screen Modules\n" + modules + "\n#%Handle Arguments", text, flags=re.DOTALL)

        #write out
        with open(self.p_project+"/"+self.script,"w") as f:
            f.write(text)
        print("Stitched: ")
        for i in stitched:
            print(f"\t{i} to {self.name}")

    def syncVersion(self) -> int:
        """Syncs the version stored in sinfo with the version in memory
        """
        with open(self.p_sinfo,"r") as f:
            info = json.load(f)
        info[self.title]["Screens"][self.name]["current"] = self.current
        with open(self.p_sinfo,"w") as f:
            json.dump(info,f)
        return 1

    def crntVersion(self) -> int:
        """Checks if the version needs to be synced and returns 1 if its synced
        """
        if not self.s_version == self.current:
            self.current = self.version
            self.syncVersion()
            return 1
        else:
            return 0

    def unload(self,root:Tk|Toplevel) -> int:
        """Unloads all elements on the screen"""
        for element in root.winfo_children():
            try:
                element.destroy()
                del element
            except: pass 
            #might fail to delete widgets that get deleted by earlier deletions
        gc.collect()

    def load(self):
        """Loads loads this screen"""
        Path(getPath()+"/"+self.script)
        os.execl(sys.executable, *(sys.executable,Path(getPath()+"/"+self.script)))
        
    def switch(self, root:Tk|Toplevel, screen:str):
        """Unloads the current screen and sets a new screen"""
        self.unload(root)
        findScreen(screen).load()

    def getModules(self, script:str=None) -> list[str]:
        """Gets a list of all modules in the screens folder"""
        if script is None: script = self.script
        path = self.p_project+"/"+script
        with open(path,"r") as file:
            modules=[]
            for line in file:
                splitline = line.split(" ")
                if splitline[0] == "from" or splitline[0] == "import":
                    if splitline[1].split(".")[0] in ["Screens", "modules"]:
                        modulename = splitline[1].replace("\n","")
                        modules.append(modulename)
                        modulepath = modulename.replace(".","/")+".py"
                        for i in self.getModules(modulepath):
                            if not i in modules:
                                modules.append(i)
        return modules
    
    def isolate(self):
        """Disabled releasing of other screens temporarily by settings them to None"""
        with open(self.p_sinfo,"r") as f:
            info = json.load(f)
            
        for i in info[self.title]["Screens"]:
            if i == self.name:
                if info[self.title]["Screens"][i]["release"] is True:
                    pass
                else:
                    print("Screen is not setup to release.")
            else:
                if info[self.title]["Screens"][i]["release"] is True:
                    info[self.title]["Screens"][i]["release"] = None

        with open(self.p_sinfo,"w") as f:
            json.dump(info,f,indent=4)

def findScreen(screenname:str)->Screen:
    """Finds a screen object from a screenname"""
    project = VINFO()
    #Load project info
    with open(project.p_sinfo,"r") as f:
        info = json.load(f)
    
    #Search project info
    if not info[project.title]["Screens"].get(screenname) is None:
        sinfo = info[project.title]["Screens"][screenname]
        return Screen(screenname,script = sinfo["script"])
    return None


from VIStk.Structures.project import *
from VIStk.Structures.VINFO import *
from VIStk.Structures.screen import *
import subprocess
import shutil
from os.path import exists
from zipfile import *
import datetime

info = {}

class Release(Project):
    """A VIS Release object"""
    def __init__(self, flag:str="",type:str="",note:str=""):
        """Creates a Release object to release or examine a releaes of a project"""
        super().__init__()
        self.type = type
        self.flag = flag
        self.note = note

        self.location = self.dist_location.replace(".",self.p_project)
        self._internal = f"{self.location}{self.title}-{self.flag}/_internal/"

    def build(self):
        """Build project spec file for release
        """
        
        #Announce Spec Creation
        print(f"Creating project.spec for {self.name}")
        
        #Ensure spec template has hidden imports
        with open(self.p_vinfo+"/Templates/spec.txt","r+") as f:
            oldspec = f.readlines()
            newspec=""
            for line in oldspec:
                if "hiddenimports" in line:
                    line = "\thiddenimports=" + str(self.hidden_imports) + ",\n"
                newspec = newspec + line
            f.seek(0)
            f.write(newspec)
            f.truncate()

        #Load Spec & Collect
        with open(self.p_vinfo+"/Templates/spec.txt","r") as f:
            spec = f.read()
        with open(self.p_vinfo+"/Templates/collect.txt","r") as f:
            collect = f.read()
        
        #Initialize locations for builds
        spec_list = []
        name_list = []
        if os.path.exists(self.p_vinfo+"/Build"):
            shutil.rmtree(self.p_vinfo+"/Build")
        os.mkdir(self.p_vinfo+"/Build")

        #Loop and Build Screens as .txt
        for i in self.screenlist:
            if i.release:
                name_list.append(i.name)
                if not i.icon == None:
                    icon = i.icon
                else:
                    icon = self.d_icon
                if str.upper(sys.platform)=="WIN32":
                    ixt = ".ico"
                else:
                    ixt = ".xbm"
                icon = icon + ixt
                spec_list.append(spec.replace("$name$",i.name))
                spec_list[-1] = spec_list[-1].replace("$icon$",icon)
                spec_list[-1] = spec_list[-1].replace("$file$",i.script)

                #Load metadata template
                with open(self.p_templates+"/version.txt","r") as f:
                    meta = f.read()

                #Update Overall Project Version
                vers = self.version.split(".")
                major = vers[0]
                minor = vers[1]
                patch = vers[2]
                meta = meta.replace("$M$",major)
                meta = meta.replace("$m$",minor)
                meta = meta.replace("$p$",patch)

                #Update Screen Version
                vers = i.s_version.split(".")
                major = vers[0]
                minor = vers[1]
                patch = vers[2]
                meta = meta.replace("$sM$",major)
                meta = meta.replace("$sm$",minor)
                meta = meta.replace("$sp$",patch)

                #Update Company Info
                if self.company != None:
                    meta = meta.replace("$company$",self.company)
                    meta = meta.replace("$year$",str(datetime.datetime.now().year))
                else:
                    meta = meta.replace("            VALUE \"CompanyName\",      VER_COMPANYNAME_STR\n","")
                    meta = meta.replace("            VALUE \"LegalCopyright\",   VER_LEGALCOPYRIGHT_STR\n","")
                    meta = meta.replace("#define VER_LEGAL_COPYRIGHT_STR     \"Copyright © $year$ $company$\\0\"\n\n","")
                
                #Update Name & Description
                meta = meta.replace("$name$",i.name)
                meta = meta.replace("$desc$",i.desc)
                
                #Write Screen Version Metadata to .txt
                with open(self.p_vinfo+f"/Build/{i.name}.txt","w") as f:
                    f.write(meta)

                #Speclist point to correct path
                spec_list[-1] = spec_list[-1].replace("$meta$",f"./Build/{i.name}.txt")
                spec_list.append("\n\n")

        #Create _a, _pyz, _exe and insert into Collect
        if sys.platform == "linux": #No Collects on Linux
            collect = ""
            for i in range(0,len(spec_list),1):
                spec_list[i] = spec_list[i].replace("exclude_binaries=True","exclude_binaries=False")
        else:
            insert = ""
            for i in name_list:
                insert=insert+"\n\t"+i+"_exe,\n\t"+i+"_a.binaries,\n\t"+i+"_a.zipfiles,\n\t"+i+"_a.datas,"
            collect = collect.replace("$insert$",insert)
            collect = collect.replace("$version$",self.name+"-"+self.flag) if not self.flag == "" else collect.replace("$version$",self.name)
            
        #Header for specfile
        header = "# -*- mode: python ; coding: utf-8 -*-\n\n\n"

        #Write Spec
        with open(self.p_vinfo+"/project.spec","w") as f:
            f.write(header)
            f.writelines(spec_list)
            f.write(collect)

        #Announce Completion
        print(f"Finished creating project.spec for {self.title} {self.flag if not self.flag =='' else 'current'}")#advanced version will improve this

    def clean(self):
        """Cleans up build environment to save space and appends to _internal"""
        #Announce Removal
        print("Cleaning up build environment")

        #Remove Build Folder
        if exists(self.p_vinfo+"/Build"):
            shutil.rmtree(self.p_vinfo+"/Build")

        #Announce Appending Screen Data
        print("Appending Screen Data To Environment")

        #Append Screen Data
        if self.flag == "":
            #Remove Pre-existing Folders for Icons, Images, & .VIS
            if exists(f"{self.location}{self.title}/Icons/"): shutil.rmtree(f"{self.location}{self.title}/Icons/")
            if exists(f"{self.location}{self.title}/Images/"): shutil.rmtree(f"{self.location}{self.title}/Images/")
            if exists(f"{self.location}{self.title}/.VIS/"): shutil.rmtree(f"{self.location}{self.title}/.VIS/")

            #Copy Project Folder for Icons, Images, & .VIS
            shutil.copytree(self.p_project+"/Icons/",f"{self.location}{self.title}/Icons/",dirs_exist_ok=True)
            shutil.copytree(self.p_project+"/Images/",f"{self.location}{self.title}/Images/",dirs_exist_ok=True)
            shutil.copytree(self.p_project+"/.VIS/",f"{self.location}{self.title}/.VIS/",dirs_exist_ok=True)
        
        else:
            #Remove Pre-existing Folders for Icons, Images, & .VIS
            if exists(f"{self.location}{self.title}-{self.flag}/Icons/"): shutil.rmtree(f"{self.location}{self.title}-{self.flag}/Icons/")
            if exists(f"{self.location}{self.title}-{self.flag}/Images/"): shutil.rmtree(f"{self.location}{self.title}-{self.flag}/Images/")
            if exists(f"{self.location}{self.title}-{self.flag}/.VIS/"): shutil.rmtree(f"{self.location}{self.title}-{self.flag}/.VIS/")

            #Copy Project Folder for Icons, Images, & .VIS
            shutil.copytree(self.p_project+"/Icons/",f"{self.location}{self.title}-{self.flag}/Icons/",dirs_exist_ok=True)
            shutil.copytree(self.p_project+"/Images/",f"{self.location}{self.title}-{self.flag}/Images/",dirs_exist_ok=True)
            shutil.copytree(self.p_project+"/.VIS/",f"{self.location}{self.title}-{self.flag}/.VIS/",dirs_exist_ok=True)

        #Announce Completion
        print(f"\n\nReleased a new{' '+self.flag+' ' if not self.flag is None else ''}build of {self.title}!")

    def newVersion(self):
        """Updates the project version, PERMANENT, cannot be undone"""
        #Split Version for Addition
        old = str(self.version)
        vers = self.version.split(".")

        #Interate Version Number
        if self.version == "Major":
            vers[0] = str(int(vers[0])+1)
            vers[1] = str(0)
            vers[2] = str(0)
        if self.version == "Minor":
            vers[1] = str(int(vers[1])+1)
            vers[2] = str(0)
        if self.version == "Patch":
            vers[2] = str(int(vers[2])+1)
        
        #Set Version Number
        self.setVersion(f"{vers[0]}.{vers[1]}.{vers[2]}")

        #Announce Completation
        print(f"Updated Version {old}=>{self.version}")

    def release(self):
        """Releases a version of your project"""
        #Check Version
        if self.type == "":
            self.newVersion()

        #Build
        self.build()

        #Announce and Update Required Tools
        print("Updating pip...")
        subprocess.call(f"python -m pip install --upgrade pip --quiet",shell=True)

        print("Updating setuptools...")
        subprocess.call(f"python -m pip install --upgrade setuptools --quiet",shell=True)

        print("Updating pyinstaller...")
        subprocess.call(f"python -m pip install --upgrade pyinstaller --quiet",shell=True)

        #Determine Binary Destination
        if sys.platform == "linux":
            destination = self.location+self.title
            if not self.flag == "": destination = destination + "-" + self.flag
        else:
            destination = self.location

        #Announce and Run PyInstaller
        print(f"Running PyInstaller for {self.title}{' ' + self.flag if not self.flag =='' else ''}")
        subprocess.call(f"pyinstaller {self.p_vinfo}/project.spec --noconfirm --distpath {destination} --log-level FATAL",shell=True,cwd=self.p_vinfo)
        
        #Clean Environment
        self.clean()

        #%Installer Generation
        #Move to Installer Build Location
        returndir = os.getcwd()
        if not returndir == self.location:
            os.chdir(self.location)

        #Create Installer
        final = "/".join(destination.split("/")[:-1])+"/"
        pendix = self.title
        if not self.flag == "": pendix = pendix + "-" + self.flag
        final = final + pendix

        #Announce and binaries.zip
        print(f"Creating binaries.zip from {final} for installer")
        shutil.make_archive(base_name="binaries",format="zip",root_dir=final)

        #Load info from binaries.zip
        archive = ZipFile('./binaries.zip','r')
        pfile = archive.open(".VIS/project.json")
        info = json.load(pfile)
        pfile.close()
        archive.close()
        title = list(info.keys())[0]

        #Get Installer Icon
        icon_file = info[title]["defaults"]["icon"]
        if sys.platform == "win32":
            icon_file = self.p_project + "/Icons/" + icon_file + ".ico"
        else:
            icon_file = self.p_project + "/Icons/" + icon_file + ".xbm"

        #Name & Compile Installer
        installer = VISROOT.replace("\\","/")+"Structures/Installer.py"
        print(f"Compiling Installer for {pendix}")
        subprocess.call(f"pyinstaller --noconfirm --onefile --add-data binaries.zip:. {'--uac-admin ' if sys.platform == 'win32' else ''}--windowed --name {pendix}_Installer --log-level FATAL --icon {icon_file} --hidden-import PIL._tkinter_finder {installer}", shell=True)

        #Move Installer to Project Root
        print("Installer completed. Moving to project root...")
        binstaller = glob.glob(f"{pendix}_Installer*",root_dir=self.location+"dist/")[0]
        if os.path.exists(self.p_project+"/"+binstaller):
            os.remove(self.p_project+"/"+binstaller)

        shutil.move(self.location+f"dist/{binstaller}",self.p_project)

        #Clean Installer Build Environment
        print("Cleaning up installer build environment...")
        shutil.rmtree(self.location+"dist/")
        shutil.rmtree(self.location+"build/")
        os.remove(self.location+"binaries.zip")
        os.remove(self.location+f"{pendix}_installer.spec")

        os.chdir(returndir)


from tkinter import *

class Window:
    """A VIS Window object"""
    def __init__(self):
        """Initializes the VIS Window"""

    def fullscreen(self,absolute:bool=False):
        if absolute is False:
            try: #On Linux
                self.wm_attributes("-zoomed", True)
            except TclError: #On Windows
                self.state('zoomed')
        else:
            self.attributes("-fullscreen", True)

    def unfullscreen(self,absolute:bool=False):
        if absolute is False:
            try: #On Linux
                self.wm_attributes("-zoomed", False)
            except TclError: #On Windows
                self.state('normal')
        else:
            self.attributes("-fullscreen", False)


import shutil
from zipfile import *
from VIStk.Objects import Root
from tkinter import ttk
from tkinter import filedialog
from tkinter import *
from PIL import Image
import PIL.ImageTk
import sys
import json
import os

#%Should be ran by the releaser
#shutil.make_archive(base_name="binaries",format="zip",root_dir="L:/WOM/PYWOM/dist/PYWOM")

#with ZipFile("L:/WOM/PYWOM/dist/binaries.zip", 'r') as f:
    #f.extractall("L:/WOM/PYWOM/dist/extracted")

#%Installer Code
#Load .VIS project info
root_location = "/".join(__file__.replace("\\","/").split("/")[:-1])

archive = ZipFile(root_location+'/binaries.zip','r')#Should find file differently
pfile = archive.open(".VIS/project.json")
info = json.load(pfile)
pfile.close()

title = list(info.keys())[0]

#%Locate Binaries
installables = []
for i in archive.namelist():
    if not any(breaker in i for breaker in ["Icons/","Images/",".VIS/","_internal"]):
        installables.append(".".join(i.split(".")[:-1]))

#%Configure Root
root = Root()

#Root Title
root.title(title + " Installer")

#Root Icon
icon_file = info[title]["defaults"]["icon"]
if sys.platform == "win32":
    icon_file = icon_file + ".ico"
else:
    icon_file = icon_file + ".xbm"

i_file = archive.open("Icons/"+icon_file)
d_icon = Image.open(i_file)
icon = PIL.ImageTk.PhotoImage(d_icon)
i_file.close()
root.iconphoto(False, icon)

#Root Geometry
root.WindowGeometry.setGeometry(width=360,height=200,align="center")
root.minsize(width=360,height=200)

#Root Layout
root.rowconfigure(1,weight=1,minsize=120)
root.rowconfigure(2,weight=1,minsize=40)
root.rowconfigure(3,weight=1,minsize=40)

root.columnconfigure(1,weight=1,minsize=180)
root.columnconfigure(2,weight=1,minsize=180)

#Scrollable frame for selection
install_frame = ttk.Frame(root,)
canvas = Canvas(install_frame,height=install_frame.winfo_height(),width=install_frame.winfo_width())
scrollbar = ttk.Scrollbar(install_frame, orient="vertical", command=canvas.yview)
install_options = ttk.Frame(canvas,height=root.winfo_height(),width=root.winfo_width())

canvas.create_window((0, 0), window=install_options, anchor="nw")

install_options.bind(
    "<Configure>",
    lambda e: canvas.configure(
        scrollregion=canvas.bbox("all")
    )
)
canvas.configure(yscrollcommand=scrollbar.set)

canvas.pack(side="left", fill="both", expand=True)
scrollbar.pack(side="right", fill="y")

install_frame.grid(row=1,column=1,columnspan=2,sticky=(N,S,E,W))

install_options.rowconfigure(0,weight=1)

install_options.columnconfigure(1,minsize=15,weight=1)
install_options.columnconfigure(2,weight=1)

#Create Checkbutton Elements
all_options = []
var_options = []
img_options = []

var_all = IntVar()

def all_state():
    """Sets the state of all the check boxes"""
    for i in var_options:
        i.set(var_all.get())

all = ttk.Checkbutton(install_options,
                      text="All",
                      variable=var_all,
                      command=all_state)
all.grid(row=0,column=1,columnspan=2,sticky=(N,S,E,W))
all.state(['!alternate'])

def is_all():
    """Checks if all of the states are selected"""
    for i in all_options:
        if var_options[all_options.index(i)].get() == 0:
            var_all.set(0)
            break
    else:
        var_all.set(1)

#Create Checkboxes
for i in installables:
    if i == "": continue
    #Configure Row
    install_options.rowconfigure(installables.index(i)+1,weight=1)

    #Resolve Installable Icon
    if info[title]["Screens"][i].get("icon") is None:
        img_options.append(PIL.ImageTk.PhotoImage(d_icon.resize((16,16))))

    else:
        icon_file = info[title]["Screens"][i]["icon"]
        if sys.platform == "win32":
            icon_file = icon_file + ".ico"
        else:
            icon_file = icon_file + ".xbm"
        
        img_options.append(PIL.ImageTk.PhotoImage(Image.open(archive.open("Icons/"+icon_file)).resize((16,16))))
        i_file.close()
    #Create Checkbox in List
    var_options.append(IntVar())
    all_options.append(ttk.Checkbutton(install_options,
                                       text=i,
                                       variable=var_options[-1],
                                       command=is_all,
                                       image=img_options[-1],
                                       compound=LEFT))
    all_options[-1].grid(row=installables.index(i)+1,column=2,sticky=(N,S,E,W))
    all_options[-1].state(['!alternate'])

#File Location
file_location = StringVar()
if sys.platform in ["win32","linux"]:
    if sys.platform=="win32":
       file_location.set("C:/Program Files")
    else:
        file_location.set("/opt")

fframe = ttk.Frame(root)
fframe.grid(row=2,column=1,columnspan=2,sticky=(N,S,E,W))

fframe.rowconfigure(1, weight=1)
fframe.columnconfigure(1,weight=1,minsize=250)
fframe.columnconfigure(2,weight=1,minsize=110)

file = ttk.Label(fframe,textvariable=file_location,relief="sunken")
file.grid(row=1,column=1,padx=2,pady=8,sticky=(N,S,E,W))

def select():
    """Select the file location"""
    selection = filedialog.askdirectory(initialdir=file_location.get(), title="Select Installation Directory")
    if not selection in ["", None]:
        file_location.set(selection)

#File Location Selection
fs = ttk.Button(fframe,
                text="Select Directory",
                command=select)
fs.grid(row=1,column=2,padx=2,pady=4,sticky=(N,S,E,W))

#Frame to beautify the controls
control = ttk.Frame(root)
control.grid(row=3,column=2,sticky=(N,S,E,W))

control.rowconfigure(1,weight=1)
control.columnconfigure(1,weight=1)
control.columnconfigure(2,weight=1)

#Close Button
close = ttk.Button(control,text="Close",command=root.destroy)
close.grid(row=1,column=1,padx=2,pady=4,sticky=(N,S,E,W))

def binstall():
    """Installs the selected binaries"""
    some=False
    for i in range(0,len(var_options),1):
        if var_options[i].get() == 1:
            some = True
            break
    if some:

        close.state(["disabled"])
        fs.state(["disabled"])
        install_options.unbind("<Configure>")
        install_options.destroy()
        
        #scrollbar.destroy()
        root.update()

        location = file_location.get()+"/"+title
        if os.path.exists(location):
            shutil.rmtree(location)
        os.mkdir(location)

        if var_all.get() == 1:
            canvas.create_text(10,10,text="Installing All...",anchor="nw")
            root.update()
            canvas.delete("all")
            archive.extractall(location)
        else:
            os.mkdir(location+"/.VIS")
            os.mkdir(location+"/Images")
            os.mkdir(location+"/Icons")
            os.mkdir(location+"/_internal")
            for file in archive.namelist():
                canvas.delete("all")
                canvas.create_text(10,10,text=f"Installing {file}...",anchor="nw")
                root.update()
                if file.startswith(".VIS/"):
                    archive.extract(file, location)
                
                canvas.delete("all")
                canvas.create_text(10,10,text=f"Installing {file}...",anchor="nw")
                root.update()
                if file.startswith("Images/"):
                    archive.extract(file, location)

                canvas.delete("all")
                canvas.create_text(10,10,text=f"Installing {file}...",anchor="nw")
                root.update()
                if file.startswith("Icons/"):
                    archive.extract(file, location)

                canvas.delete("all")
                canvas.create_text(10,10,text=f"Installing {file}...",anchor="nw")
                root.update()
                if file.startswith("_internal/"):
                    archive.extract(file, location)

            for i in range(0,len(var_options),1):
                if var_options[i].get() == 1:
                    for file in archive.namelist():
                        if file.startswith(installables[i]):
                            canvas.delete("all")
                            canvas.create_text(10,10,text=f"Installing {file}...",anchor="nw")
                            root.update()
                            archive.extract(file, location)

        root.destroy()

#Install Button
install = ttk.Button(control, text="Install",command=binstall)
install.grid(row=1,column=2,padx=2,pady=4,sticky=(N,S,E,W))

root.mainloop()


from tkinter import *
from tkinter import ttk
from VIStk.Objects import Root
import sys

root = Root()
root.title('New VIS Project')

def do_focus(event=None):
    x,y = root.winfo_pointerxy()
    root.winfo_containing(x,y).focus()

root.bind("<1>", do_focus)


from tkinter import *
from typing import Literal

query = Tk()
try: #On Linux
    query.wm_attributes("-zoomed", True)
except TclError: #On Windows
    query.state('zoomed')
query.update()
global hs, ws
ws = query.winfo_width()-2#Unclear about this offset
hs = query.winfo_height()-9#Might be operating system specific
query.destroy()
#print(f"Screen has usable size of {ws}x{hs}")

class WindowGeometry():
    """Handles geometry relations and sizing/resizing for windows"""
    def __init__(self,window:Tk|Toplevel):
        """Creates a window geometry object and automatically extracts the size and location
        
        Args:
            window (Tk|Toplevel): The window to access geometry of
        """
        self.window:Tk|Toplevel = window
        self.getGeometry()
        window.WindowGeometry = self

    def getGeometry(self, respect_size:bool=False):
        """Sets the internal geometry of object to match the window"""
        geo_str = self.window.geometry()
        geo_list = geo_str.split("x")
        ng_list = [int(geo_list[0])]
        for i in geo_list[1].split("+"):
            ng_list.append(int(i))
        
        if respect_size:
            self.geometry = [self.window.winfo_width(), self.window.winfo_height(), ng_list[2], ng_list[3]]
        else:
            self.geometry = ng_list

    def stripGeometry(self,objects:tuple[Literal["w","h","x","y"]]|Literal["all"]):
        """Returns integer values of the requested items"""
        geo_str = self.window.geometry()
        geo_list = geo_str.split("x")
        ng_list = [int(geo_list[0])]
        for i in geo_list[1].split("+"):
            ng_list.append(int(i))
        
        geo_list = []
        if objects == "all":
            geo_list = ng_list
        else:
            if "w" in objects: geo_list.append(ng_list[0])
            if "h" in objects: geo_list.append(ng_list[1])
            if "x" in objects: geo_list.append(ng_list[2])
            if "y" in objects: geo_list.append(ng_list[3])

        return geo_list


    def setGeometry(self,width:int=None,height:int=None,x:int=None,y:int=None,align:Literal["center","n","ne","e","se","s","sw","w","nw"]=None,size_style:Literal["pixels","screen_relative","window_relative"]=None,window_ref:Tk|Toplevel=None):
        """Sets the geometry of the window"""
        global hs, ws
        ox, oy = 0, 0

        if width is None: width = self.geometry[0]
        if height is None: height = self.geometry[1]

        #Check if aligning or using coordinates
        if align is None:
            if x is None: x = self.geometry[2]
            if y is None: y = self.geometry[3]
        else:
            x = None
            y = None
        
        #No adjustment needs to be made if pixels are given

        if size_style == "window_relative":
            if not window_ref is None:
                _ws = window_ref.winfo_width()
                _hs = window_ref.winfo_height()

                if not _ws is None: ws = _ws
                if not _hs is None: hs = _hs

                geo_ref = WindowGeometry(window_ref).stripGeometry(("x","y"))
                
                ox = geo_ref[0]
                oy = geo_ref[1]

            #Will always hand over relative sizing to screen relative
            size_style = "screen_relative"

        if size_style == "screen_relative":
            if not width == self.geometry[0]:
                width = ws*width/100
                
            if not height == self.geometry[1]:
                height = hs*height/100

        if not align is None:
            match align:
                case "center":
                    x = ws/2 - width/2 + ox
                    y = hs/2 - height/2 + oy
                case "n":
                    x = ws/2 - width/2 + ox
                    y = 0 + oy
                case "ne":
                    x = ws - width + ox
                    y = 0 + oy
                case "e":
                    x = ws - width + ox
                    y = hs/2 - height/2 + oy
                case "se":
                    x = ws - width + ox
                    y = hs - height + oy
                case "s":
                    x = ws/2 - width/2 + ox
                    y = hs - height + oy
                case "sw":
                    x = 0 + ox
                    y = hs - height + oy
                case "w":
                    x = 0 + ox
                    y = hs/2 - height/2 + oy
                case "nw":
                    x = 0 + ox
                    y = 0 + oy

        self.geometry = [int(width), int(height), int(x-7), int(y)]
        self.window.geometry('%dx%d+%d+%d' % tuple(self.geometry))